﻿using System;
using Helios.Models;
using Npgsql;

namespace Helios.Source
{
	public class ConfiguratorService
	{
		private readonly NpgsqlConnection _connection;
		private readonly List<SolarPanel> _solarPanels;
		private readonly SunElevation _maxElevation;

		public ConfiguratorService(string connectionString)
		{
			_connection = new NpgsqlConnection(connectionString);

			this._solarPanels = GetSolarPanels();
			this._maxElevation = GetSunElevation();
		}

		public ConfiguratorResult GetResults(ConfiguratorInput input)
        {
			ConfiguratorResult result = new ConfiguratorResult();

			foreach (Roof roof in input.Roofs)
			{
				RoofResults roofResult = new RoofResults();

				roofResult.RoofAngle = roof.Elevation;
				roofResult.RoofArea = roof.Area;

				foreach (SolarPanel panel in _solarPanels)
				{
					PanelPerformance performance = new PanelPerformance();

					int summerElevationAngle = roof.GetElevationAngle(_maxElevation.SummerElevation);
					int winterElevationAngle = roof.GetElevationAngle(_maxElevation.WinterElevation);

					performance.RoofArea = roof.Area;
					performance.RoofAngle = roof.Elevation;
					performance.PowerRequired = input.PowerRequired;
					performance.MaximumPower = input.MaximumPower;

					performance.PanelEfficiency = panel.Efficiency * 100;
					performance.InstallationCost = panel.GetInstallationCost(roof.Area);
					performance.PowerGeneratedSummer = Math.Round(panel.GetPowerOutput(1000, roof.Area, summerElevationAngle), 2);
					performance.PowerGeneratedWinter = Math.Round(panel.GetPowerOutput(1000, roof.Area, winterElevationAngle), 2);
					performance.GeneratesRequiredPower = GeneratesRequiredPower(input.PowerRequired, performance.PowerGeneratedWinter);
					performance.ExceedsMaximumPower = ExceedsPowerLimit(input.MaximumPower, performance.PowerGeneratedWinter);
					performance.WithinBudget = WithinBudget(input.Budget, performance.InstallationCost);
					performance.CostPerWatt = Math.Round(panel.GetCostPerWatt(), 2);

					if (performance.RoofAngle != 0)
					{
						roofResult.Panels.Add(performance);
					}
				}

				if (roofResult.Panels.Count != 0)
                {
					result.RoofResults.Add(roofResult);
                }
			}			

			return result;
        }

		private bool WithinBudget(int budget, double installationCost)
		{
			return installationCost < budget;
		}

		private bool GeneratesRequiredPower(int requiredPower, double powerGenerated)
        {
			return powerGenerated >= requiredPower;
        }

		private bool ExceedsPowerLimit(int maxPower, double powerGenerated)
		{
			return powerGenerated >= maxPower;
		}


		private List<SolarPanel> GetSolarPanels()
        {
			List<SolarPanel> solarPanels = new List<SolarPanel>();

            _connection.Open();

            NpgsqlCommand cmd = new NpgsqlCommand("select efficiency, area_cost from solar_panel", _connection);

            using var reader = cmd.ExecuteReader();

			while (reader.Read())
            {
				solarPanels.Add(new SolarPanel
				{
					Efficiency = double.Parse(reader["efficiency"].ToString()),
					CostPerMetreSquared = double.Parse(reader["area_cost"].ToString())
				});
            }

            _connection.Close();

			return solarPanels;
        }

		private SunElevation GetSunElevation()
        {
			SunElevation elevation = new SunElevation();

			_connection.Open();

			NpgsqlCommand cmd = new NpgsqlCommand("select  \"time\", summer_eleveation, winter_eleveation from sun_elevation where \"time\" = '12:00'", _connection);

			using var reader = cmd.ExecuteReader();

			if (reader.Read())
			{
				elevation.Time = DateTime.Parse(reader["time"].ToString());
				elevation.SummerElevation = int.Parse(reader["summer_eleveation"].ToString());
				elevation.WinterElevation = int.Parse(reader["winter_eleveation"].ToString());	
			}

			_connection.Close();

			return elevation;
		}
	}
}

